<?php
/**
 * This file is part of the official Amazon Checkout extension for Magento 1.x
 *
 * (c) 2020 creativestyle GmbH. All Rights reserved
 *
 * Distribution of the derivatives reusing, transforming or being built upon
 * this software, is not allowed without explicit written permission granted
 * by creativestyle GmbH
 *
 * @category   Creativestyle
 * @package    Creativestyle_AmazonCheckout
 * @copyright  2020 creativestyle GmbH
 * @author     Marek Stefan <ticket@creativestyle.de>
 */

class Creativestyle_AmazonCheckout_Model_Service_Login
{
    const AMAZON_USER_ID_ATTRIBUTE = 'amazon_user_id';

    /**
     * Return instance of Amazon Pay API adapter
     *
     * @return Creativestyle_AmazonCheckout_Model_Api
     */
    protected function _getApi()
    {
        /** @var Creativestyle_AmazonCheckout_Model_Api $api */
        $api = Mage::getSingleton('amazonpay/api');
        return $api;
    }

    /**
     * @return Mage_Customer_Model_Session
     */
    protected function _getSession()
    {
        /** @var Mage_Customer_Model_Session $session */
        $session = Mage::getSingleton('customer/session');
        return $session;
    }

    /**
     * Return instance of Amazon Pay helper
     *
     * @return Creativestyle_AmazonCheckout_Helper_Data
     */
    protected function _getAmazonHelper()
    {
        /** @var Creativestyle_AmazonCheckout_Helper_Data $helper */
        $helper = Mage::helper('amazonpay');
        return $helper;
    }

    /**
     * @return Mage_Checkout_Model_Session
     */
    protected function _getCheckoutSession()
    {
        /** @var Mage_Checkout_Model_Session $session */
        $session = Mage::getSingleton('checkout/session');
        return $session;
    }

    /**
     * @return int|string|null
     * @throws Mage_Core_Model_Store_Exception
     */
    protected function _getWebsiteId()
    {
        return Mage::app()->getStore()->getWebsiteId();
    }

    /**
     * @param $buyerId
     * @return Mage_Customer_Model_Customer|null
     * @throws Mage_Core_Model_Store_Exception
     */
    protected function _getCustomerByBuyerId($buyerId)
    {
        /** @var Mage_Customer_Model_Customer $customer */
        $customer = Mage::getModel('customer/customer');

        /** @var Mage_Customer_Model_Resource_Customer_Collection $collection */
        $collection = $customer->getCollection()
            ->addAttributeToFilter(self::AMAZON_USER_ID_ATTRIBUTE, $buyerId)
            ->setPageSize(1);

        if ($customer->getSharingConfig()->isWebsiteScope()) {
            $collection->addAttributeToFilter('website_id', $this->_getWebsiteId());
        }

        if ($collection->count()) {
            return $collection->getFirstItem();
        }

        return null;
    }

    /**
     * @param Mage_Customer_Model_Customer $customer
     */
    protected function _setCustomerAsLoggedIn(Mage_Customer_Model_Customer $customer)
    {
        /**
         * The following operations on the quote model circumvent guest cart
         * items removal, invoked after the successful authentication within
         * the Magento session. It was introduced as a security fix in:
         *   - CE 1.9.4.2
         *   - EE 1.14.4.2
         *   - SUPEE-11155 patch
         *
         * @var Mage_Sales_Model_Quote $quote
         */
        $quote = $this->_getCheckoutSession()->getQuote();
        if (!$quote->getCustomerId()) {
            $quote->setCustomerId($customer->getId())->save();
        }

        $this->_getSession()->setCustomerAsLoggedIn($customer);
    }

    protected function _getCustomerByEmail($email)
    {
        /** @var Mage_Customer_Model_Customer $customer */
        $customer = Mage::getModel('customer/customer');

        if ($customer->getSharingConfig()->isWebsiteScope()) {
            $customer->setData('website_id', $this->_getWebsiteId());
        }

        $customer->loadByEmail($email);

        if ($customer->getId()) {
            return $customer;
        }

        return null;
    }

    /**
     * @return Mage_Customer_Model_Customer
     * @throws Mage_Core_Model_Store_Exception
     * @throws Exception
     */
    protected function _createCustomer(Creativestyle_AmazonCheckout_Model_Api_Buyer $buyer)
    {
        $customerName = $this->_getAmazonHelper()->splitCustomerName($buyer->name);

        /** @var Mage_Customer_Model_Customer $customer */
        $customer = Mage::getModel('customer/customer');
        $password = $customer->generatePassword(8);
        $customer->setId(null)
            ->setSkipConfirmationIfEmail($buyer->email)
            ->setFirstname($customerName->getFirstname())
            ->setLastname($customerName->getLastname())
            ->setEmail($buyer->email)
            ->setPassword($password)
            ->setPasswordConfirmation($password)
            ->setConfirmation($password)
            ->setAmazonUserId($buyer->buyerId);

        if ($customer->getSharingConfig()->isWebsiteScope()) {
            $customer->setData('website_id', $this->_getWebsiteId());
        }

        // validate customer
        $validation = $customer->validate();
        if ($validation !== true && !empty($validation)) {
            $validation = implode(", ", $validation);
            throw new Creativestyle_AmazonCheckout_Exception(
                sprintf('Error while creating customer account: %s', $validation)
            );
        }

        $customer->save();

        return $customer;
    }

    public function auth($buyerToken)
    {
        $buyer = $this->_getApi()->getBuyer($buyerToken);
        $customer = $this->match($buyer);
        $this->_setCustomerAsLoggedIn($customer);
        return $customer;
    }

    public function match(Creativestyle_AmazonCheckout_Model_Api_Buyer $buyer)
    {
        $customer = $this->_getCustomerByBuyerId($buyer->buyerId);

        if (null === $customer) {
            $customer = $this->_getCustomerByEmail($buyer->email);

            if (null !== $customer) {
                $customer->setAmazonCustomerId($buyer->buyerId);
                $customer->save();
            }
        }

        if (null === $customer) {
            $customer = $this->_createCustomer($buyer);
        }

        return $customer;
    }
}
